VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "EnhancedMenu"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Collection" ,"UserDefinedCollection"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
''' Enhanced Menu v2.3

''' Owner-drawn menu control for all Windows editions.
''' Frame-window menu-bar style menu object.

''' Copyright (C) 2000 Nathan Moschkin.
''' All Rights Reserved.

''' Free to copy and distribute, intact.  Give credit to me and anyone else
''' who may edit this code.

''' Please inquire before using any of this code or the DLL control commercially.
''' An OCX version of the menu with added features will be available soon.
''' OCX package coming in Version 4.0

''' E-Mail questions or comments to nmosch@tampabay.rr.com

Option Explicit

''' Private variables that represent public or read-only properties

'' Connection property

Private m_Connection As MDIConnector

'' Identification

Private m_MenuId As Long

'' Handle

Private m_hMenu As Long

'' Window-related

Private m_OldWndProc As Long

Private m_OldWndMenu As Long

Private m_hWnd As Long

'' Information about this menu stored in Win32 standard format

Private m_ItemInfo As MENUITEMINFO

'' Collection to store child items

Private m_Col As Collection

'' Accelerator collection

Private m_Accel As AccelCol

'' Window menu property (for MDI forms)

Private m_WindowList As Submenu

'' Default Font property

Private m_Font As StdFont

' The ItemSelect event

Public Event ItemSelect(MenuObject As MenuItem)

' Menu types.

Public Enum EnhancedMenuTypeConstants
    emPopupMenu = &H1
    emBarMenu = &H2
End Enum


''' Window-related properties
'''

' Owner Window Handle

Public Property Get hwnd() As Long
    hwnd = m_hWnd
End Property

' Old window procedure

Public Property Get OldWndProc() As Long
    OldWndProc = m_OldWndProc
End Property

' Old window menu

Public Property Get OldWndMenu() As Long
    OldWndMenu = m_OldWndMenu
End Property

'''


''' Menu-related properties
'''

' Menu handle

Public Property Get Handle() As Long
    Handle = m_hMenu
End Property

''' The Subclass method may need to set this variable externally

Friend Property Let Handle(ByVal vData As Long)
    On Error Resume Next
    
    If m_hMenu <> 0 Then Destroy
    
    m_hMenu = vData
    
    If m_hMenu <> 0 Then
        If Not g_MenuCol("H" + Hex(m_hMenu)) Is Nothing Then
            g_MenuCol.Remove "H" + Hex(m_hMenu)
        End If
        
        g_MenuCol.Add Me, "H" + Hex(m_hMenu)
    End If
    
End Property

''' The Menu ID is assigned when the menu object is created, but
''' re-assigned when a menu is added to the collection.
''' The GetNewCtrlId function generates a CtrlID at the top most level
''' of the menu heirarchy.

''' The menu ID for a menu bar toplevel menu is arbitrary, it is not
''' physically assigned in Windows.

Public Property Get MenuId() As Long
    MenuId = m_MenuId
End Property

Friend Property Let MenuId(ByVal vData As Long)
    m_MenuId = vData
End Property

'' Keyboard accelerator collection property

Public Property Get Accelerators() As AccelCol
    On Error Resume Next
    
    Set Accelerators = m_Accel

End Property

'''

' Font property

Public Property Get Font() As StdFont

    Set Font = m_Font
    
End Property

Public Property Set Font(IFont As StdFont)

    Set m_Font = IFont
    
    If m_hWnd <> 0 Then
        DrawMenuBar m_hWnd
    End If
    
End Property


' Function to set the MENUITEMINFO structure
' from the owning Item's MENUITEMINFO.
' The item's info is valid and updated
' every time the parent item's SetCtrlState or GetCtrlState functions are called.
' This is to ensure that the Submenu has the proper data variables to pass to
' the menu handler when creating new sub items.

Friend Sub SetItemInfo(v_Info As MENUITEMINFO)
    
    CopyMemory m_ItemInfo, v_Info, Len(m_ItemInfo)
    
End Sub

'' Instructs the object to fire the ItemSelect event if the object is delcared 'WithEvents'
'' This function is called when an item is selected by the window subclassing procedure in
'' the m_EnhancedMenu module.

Friend Sub MenuSelectEvent(ByVal wParam As Long)
    Dim v_Menu As MenuItem
    
    Set v_Menu = SearchRecursive(Me, wParam)
    
    On Error Resume Next
    
    If Not v_Menu Is Nothing Then
    
        RaiseEvent ItemSelect(v_Menu)
    
    End If
End Sub

'' This property is constant in an EnhancedMenu

Public Property Get SuperParent() As EnhancedMenu
    Set SuperParent = Me
End Property

'''''' Public properties and methods relating to the menu item collection

'' Add Method

Public Function Add(ByVal sCaption As String, Optional Picture As StdPicture, Optional ByVal sKey As String, Optional ByVal InsertAfter As Variant = -1) As MenuItem
    On Error Resume Next
    
    Dim v_Info As MENUITEMINFO, v_Flags As Long, v_Str As String
    Dim v_NewItem As New MenuItem, v_InsertAfter As MenuItem
    Dim v_spHandle As Long
    v_Info.cbSize = Len(v_Info)
        
    If m_MenuId = 0 Then Exit Function
    
    ''' Create this menu as necessary...
    
    If m_hMenu = 0 Then Create
                           
    ''' Set the parent of the new item to this instance.
    
    Set v_NewItem.Parent = Me
    
    ''' If there is no key, it is the hexidecimal value of the control id.
    
    If sKey = "" Then
        v_NewItem.Key = "H" + Hex(v_NewItem.MenuId)
    Else
        v_NewItem.Key = sKey
    End If
    
    ''' Menu items are strings. :)  '' Owner drawn
    
    v_Flags = MF_OWNERDRAW
    
    ''' Use the old AppendMenu to quickly add a menu item.
    
    If VarType(InsertAfter) = vbString Then
        Set v_InsertAfter = m_Col(InsertAfter)
    Else
        If InsertAfter <> -1 Then
            Set v_InsertAfter = Me.Search(InsertAfter)
        End If
    End If
    
    v_spHandle = m_hMenu
    
    If Not v_InsertAfter Is Nothing Then
        InsertMenu m_hMenu, v_InsertAfter.MenuId, 0, CLng(v_NewItem.MenuId), v_spHandle
    Else
        AppendMenu m_hMenu, v_Flags, v_NewItem.MenuId, v_spHandle
    End If
    
    ''' Now set the item's properties so that they are picked up
    ''' by the physical control...
    
    v_NewItem.Caption = sCaption
    v_NewItem.ClearHighlight = True
    
    If Not Picture Is Nothing Then
        Set v_NewItem.Picture = Picture
    End If
    
    ''' Add the item to the collection
    
    m_Col.Add v_NewItem, v_NewItem.Key
        
    ' Since this is a menu bar object, we redraw the menu bar to reflect the
    ' changes.
    
    If (m_hWnd <> 0) And Not Connected Then
        DrawMenuBar m_hWnd
    End If
    
    Set Add = v_NewItem
    Set v_NewItem = Nothing
    
End Function

'' Get an item

Public Property Get Item(varIndexKey As Variant) As MenuItem
Attribute Item.VB_UserMemId = 0
Attribute Item.VB_MemberFlags = "200"
  On Error Resume Next
  Set Item = m_Col(varIndexKey)
End Property

'' Get the item count

Public Property Get Count() As Long
    Count = m_Col.Count
End Property

'' Destroy and remove a menu item

''' Destroy the sub menus first, or we will generate an exception.
''' You must destroy the menus before terminating the class.

Public Sub Remove(varIndexKey As Variant)
    On Error Resume Next
    
    If Not m_Col(varIndexKey) Is Nothing Then
        m_Col(varIndexKey).ReleaseWindowHandle
        Set m_Col(varIndexKey).Connection = Nothing
        
        m_Col.Remove varIndexKey
    End If
    
End Sub

'' Destroy all sub menus and create a new collection

''' Destroy the sub menus first, or we will generate an exception.
''' You must destroy the menus before terminating the class.

Public Sub Clear()
    On Error Resume Next
    
    Dim v_Item As MenuItem
    
    If m_Col.Count > 0 Then
        For Each v_Item In Me
            v_Item.Destroy
        Next v_Item
           
    End If
    
    Set m_Col = New Collection
End Sub

'' NewEnum function

Public Property Get NewEnum() As IUnknown
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
    'this property allows you to enumerate
    'this collection with the For...Each syntax
    
    Set NewEnum = m_Col.[_NewEnum]
End Property

Friend Property Get Col() As Collection
    Set Col = m_Col
End Property


''' Properties and methods used in connecting to MDI applications

'' Connection property (applies to usage in MDI applications)

Public Property Get Connection() As MDIConnector
    Set Connection = m_Connection
End Property

Friend Property Set Connection(v_Data As MDIConnector)
    Set m_Connection = v_Data
End Property

Public Property Get Connected() As Boolean
    On Error Resume Next
    
    If Not m_Connection Is Nothing Then
        Connected = True
    Else
        Connected = False
    End If
End Property

'' WindowList property (for MDI forms)

Public Property Get WindowList() As Submenu
    Set WindowList = m_WindowList
End Property

Public Sub SetWindowList(Menu)
    Dim v_hMenu As Long
    Dim v_Item As MenuItem

    Select Case TypeName(Menu)
    
        Case "Submenu"
                    
            If Menu.SuperParent Is Me Then
                Set m_WindowList = Menu
            End If
            
        Case "String", "Integer", "Long"
        
            Set m_WindowList = m_Col(Menu).Submenu
            
    End Select
     
    v_hMenu = m_WindowList.Handle
     
    For Each v_Item In m_Col
    
        If v_Item.Submenu.Handle <> v_hMenu Then
            v_Item.Submenu.WindowList = False
        Else
            v_Item.Submenu.WindowList = True
        End If
        
    Next v_Item

    If Connected Then
    
        g_WindowListHandle = v_hMenu
    
        RefreshChildWindows m_Connection.hWndFrame
        
        g_IsSetting = True
        
        SendMessage m_Connection.hWndClient, WM_MDISETMENU, 0&, v_hMenu
        SendMessage m_Connection.hWndClient, WM_MDIREFRESHMENU, 0&, 0&
        
        g_IsSetting = False
    
        DrawMenuBar m_Connection.hWndFrame
        
    End If

End Sub

'' Search method passes values on to the global SearchRecursive routine

Public Function Search(varIndexKey As Variant) As MenuItem

    ''' See SearchRecursive method in m_EnhancedMenu
    Set Search = SearchRecursive(Me, varIndexKey)
    
End Function


'' Method to cause this object to take over the drawing responsibilities
'' for an existing menu.  This object becomes, then, an 'interface' to that
'' menu, and the internal m_hMenu identifier is changed to the menu that is
'' being subclassed.


'' Method to cause this object to take over the drawing responsibilities
'' for an existing menu.  This object becomes, then, an 'interface' to that
'' menu, and the internal m_hMenu identifier is changed to the menu that is
'' being subclassed.

Public Sub Subclass(ByVal hwnd As Long)
    On Error Resume Next
        
    Dim v_Item As MenuItem, v_Sub As Submenu
    Dim i_Count As Long, i As Long
    Dim v_Info As MENUITEMINFOSTRING, v_Str As String
    Dim Flags As Long, vl_Info As MENUITEMINFO

    Dim j As Integer, s As String

    '' Here is a case where we want our menu destroyed completely, first.
    
    Destroy
    
    m_hMenu = GetMenu(hwnd)
    
    SetWindowHandle hwnd
    
    CopyMenuToObject Me, m_hMenu, True
    
End Sub

'' Method to copy the contents of another menu into this one

Public Sub CopyMenu(ByVal hMenu As Long)
    
    CopyMenuToObject Me, hMenu
    
    If m_hWnd <> 0 Then
        DrawMenuBar m_hWnd
    End If

End Sub


'' Release the window handle from the subclassing procedure and
'' remove associations between this instance and the assigned window

Public Sub ReleaseWindowHandle()

    On Error Resume Next
        
    If m_hWnd <> 0 Then
        
        If Not Connected Then
        
            ' Re-instate the old window procedure (un-hook)
            
            SetWindowLong m_hWnd, GWL_WNDPROC, m_OldWndProc
            
            ' Remove the handle from the handle array
                    
            RemoveHandle m_hWnd
        
        End If
        
        m_hWnd = 0
                    
    End If
    
End Sub

    ' We only want top level menus to hook to windows.
    ' popup menus can hook through the Popup subroutine,
    ' or directly with a call to TrackPopupMenu.
    
Public Sub SetWindowHandle(ByVal hwnd As Long, Optional ByVal CopyOldMenu As Boolean = False)
    On Error Resume Next
        
    Dim v_Data As Long, v_DataEx As Long
    
    Dim v_Obj As Object
        
        ' cannot assign without first releasing.
        
    If m_hWnd <> 0 Then
        Me.ReleaseWindowHandle
    End If
    
    m_hWnd = hwnd
            
    If Not Connected Then
    
        m_OldWndMenu = GetMenu(m_hWnd)
        
        If m_OldWndMenu <> m_hMenu Then
            SetMenu m_hWnd, m_hMenu
                
            If CopyOldMenu = True Then
                CopyMenuToObject Me, m_OldWndMenu
            End If
        Else
            m_OldWndMenu = 0
        End If
                
        If GetOldWndProc(m_hWnd) = 0 Then
            m_OldWndProc = SetWindowLong(m_hWnd, GWL_WNDPROC, AddressOf MenuWndProc)
            AddHandle m_hWnd, m_OldWndProc, m_hMenu
        End If
        
        DrawMenuBar m_hWnd
        
    End If
            
    If g_MenuCol("H" + Hex(m_hMenu)) Is Nothing Then
            
        g_MenuCol.Add Me, "H" + Hex(m_hMenu)
    
    ElseIf Not g_MenuCol("H" + Hex(m_hMenu)) Is Me Then
    
        g_MenuCol.Remove "H" + Hex(m_hMenu)
        g_MenuCol.Add Me, "H" + Hex(m_hMenu)
    
    End If
        
End Sub
   
'' Destroy is called internally by the Class_Terminate and Subclass Methods
'' It also can be called externally (e.g. when an error has occured, or to quickly
'' delete the menu without having to unload the instance.)

Public Sub Destroy()
    Dim v_Dum As Object
    
        On Error Resume Next
    
    ReleaseWindowHandle
                        
    For Each v_Dum In Me
        v_Dum.Destroy
    Next v_Dum
                        
    If m_hMenu <> 0 Then
        Set v_Dum = g_MenuCol("H" + Hex(m_hMenu))
        
        If Not v_Dum Is Nothing Then
            g_MenuCol.Remove "H" + Hex(m_hMenu)
        End If
    
        DestroyMenu m_hMenu
    End If
    
    Clear
    
End Sub

Public Sub Create()
    On Error Resume Next
       
    If m_hMenu <> 0 Then Exit Sub
    
    m_hMenu = CreateMenu
    
    If g_MenuCol("H" + Hex(m_hMenu)) Is Nothing Then
        g_MenuCol.Add Me, "H" + Hex(m_hMenu)
    End If
    
    If (m_hWnd <> 0) And Not Connected Then
        SetMenu m_hWnd, m_hMenu
        DrawMenuBar m_hWnd
    End If
    
End Sub

'' Private Subroutines called when an instance is created or destroyed

Private Sub Class_Initialize()
    On Error Resume Next
    Dim v_Obj As Long, v_Data As LOGFONT
    
    Set m_Col = New Collection
        
    m_ItemInfo.cbSize = g_InfoSize
    
    ' Create a new accelerator collection

    Set m_Accel = New AccelCol
                
    ' Get new global identifier.  Menu IDs are unique throughout a thread
        
    m_MenuId = GetNewCtrlId
    
    '' Create a menu handle, add it to the menu array.
            
    Create
        
End Sub

Private Sub Class_Terminate()
    
  ' Automatically destroy the whole thing on terminate.
  '' (See notes in 'Destroy' method, above)
    
    ReleaseWindowHandle
    Destroy
    
  '' It is standard to clear collections on termination
    
    Set m_Col = Nothing
    Set m_Accel = Nothing
    
End Sub


