Attribute VB_Name = "m_Subclass"
''' Enhanced Menu v2.3

''' Owner-drawn menu drawing and callback processing.

''' Copyright (C) 2000 Nathan Moschkin.
''' All Rights Reserved.

''' Free to copy and distribute, intact.  Give credit to me and anyone else
''' who may edit this code.

''' Please inquire before using any of this code or the DLL control commercially.
''' An OCX version of the menu with added features will be available soon.

''' E-Mail questions or comments to nmosch@tampabay.rr.com

Option Explicit

Public g_IsSetting As Boolean

'' Check states.

Public Const DRWCHK_NORMAL = 0
Public Const DRWCHK_SELECTED = 1
Public Const DRWCHK_DISABLED = 2

'' routine to draw a check mark or radio bullet.

Public Sub DrawCheck(ByVal hDC As Long, _
                       rcBounds As RECT, _
                       ByVal state As Integer, _
                       ByVal Radio As Boolean _
                       )
                          
    Dim p As Long, lp As POINTAPI, op As Long
    Dim x As Long, y As Long, v_Brush As Long, v_OldBrush As Long, v_lb As LOGBRUSH
    
    ' get the draw state.
    
        ' Selected item
    If (state = DRWCHK_SELECTED) Then
        
        ' Create a pen, and set brush parameters in case we need
        ' a brush (for a bullet instead of a check
        
        ' Make it thick enough to be seen.
        
        Select Case g_WinVersion
        
            Case Windows2000, WindowsME:
                p = CreatePen(6, 2, GetSysColor(COLOR_HIGHLIGHTTEXT))
            Case Else
                p = CreatePen(6, 1, GetSysColor(COLOR_HIGHLIGHTTEXT))
            
        End Select
        
        v_lb.lbColor = GetSysColor(COLOR_HIGHLIGHTTEXT)
    
        ' Disabled/grayed item
    ElseIf (state = DRWCHK_DISABLED) Then
        
        Select Case g_WinVersion
        
            Case Windows2000, WindowsME:
                p = CreatePen(6, 2, GetSysColor(COLOR_GRAYTEXT))
            
            Case Else
                p = CreatePen(6, 1, GetSysColor(COLOR_GRAYTEXT))
            
        End Select
        
        v_lb.lbColor = GetSysColor(COLOR_GRAYTEXT)
    
        ' Normal item
    ElseIf (state = DRWCHK_NORMAL) Then
        
        Select Case g_WinVersion
        
            Case Windows2000, WindowsME:
                p = CreatePen(6, 2, 0&)
            
            Case Else
                p = CreatePen(6, 1, 0&)
            
        End Select
        
            
        v_lb.lbColor = &H0&
    
    End If
    
    ' If it's not a Radio bullet it's a check mark.
    ' Painting the check mark was a logical deduction coordinates
    ' assuming we're given a square, not a rectangle.
    
    ' Painting user-drawn controls is 50% feel, anyways.  :-)
    
    If Not Radio Then
    
                
        ' Select the pen, save the old pen.
        
        op = SelectObject(hDC, p)
        
        ' Get the dimension of the square
        x = rcBounds.Right - rcBounds.Left
        x = x / 3   ' Start from a third of the way across to
                    ' 2/3's the way across.
        
        y = rcBounds.Bottom - rcBounds.Top
        y = y / 2   ' start from the midpoint y coordinate.
        
        ' MoveToEx moves the pen drawing origin on the canvas.
        
        MoveToEx hDC, rcBounds.Left + x, rcBounds.Top + y, lp
        
        x = (rcBounds.Right - rcBounds.Left) / 2
        
        y = (rcBounds.Bottom - rcBounds.Top)
        y = (y / 3)
        
        ' We are somewhere, press on the pen and go somewhere.
        ' LineTo draws from the current origin on the DC to the specified point.
        
        ' Down swing.
        LineTo hDC, rcBounds.Left + x, rcBounds.Bottom - y
        
        x = (rcBounds.Right - rcBounds.Left) / 3
        
        y = (rcBounds.Bottom - rcBounds.Top)
        y = (y / 3.5)
        
        'Up swing.
        LineTo hDC, rcBounds.Right - x, rcBounds.Top + y
        
        ' Reselect the old object and delete the pen.
        
        SelectObject hDC, op
        DeleteObject p
    
    Else
        
        ' Select the brush, use the current pen ( we assume it's the background pen )
        
        v_Brush = CreateBrushIndirect(v_lb)
        v_OldBrush = SelectObject(hDC, v_Brush)
        
        x = (rcBounds.Right - rcBounds.Left) / 2
        x = x - (x / 3)
            
        ' The bounding rectangle has to be tuned specifically to the way that
        ' the rectangle was figured in the DrawItem function.  If that value changes,
        ' then these offset values will also change.
            
        Ellipse hDC, rcBounds.Left + x + 2, rcBounds.Top + x + 2, rcBounds.Right - x, rcBounds.Bottom - x
        
        ' set the old canvas brush and destroy the brush.
        
        SelectObject hDC, v_OldBrush
        DeleteObject v_Brush
        
    End If
    
End Sub

' Draw a Thin 3D etch around a rectangular area using the system button
' shadow and button highlight colors.

Public Sub Draw3dSq(ByVal hDC As Long, rcBounds As RECT, Optional ByVal Reversed As Boolean)
    Dim p As Long, lp As POINTAPI, op As Long
    
    ' Create the first pen for the Top and Left edges.
    
    If Reversed Then
        
        ' The reversed option provides the inversed, or 'pressed down' effect.
        p = CreatePen(6, 1, GetSysColor(COLOR_BTNSHADOW))
    Else
        p = CreatePen(6, 1, GetSysColor(COLOR_BTNHIGHLIGHT))
    End If
    
    ' Select the pen.
    
    op = SelectObject(hDC, p)
    
    ' Set the origin and draw a line.
        
    MoveToEx hDC, rcBounds.Right, rcBounds.Top, lp
    LineTo hDC, rcBounds.Left, rcBounds.Top
    LineTo hDC, rcBounds.Left, rcBounds.Bottom
        
    ' set the old pen in order to delete the first pen.
    
    SelectObject hDC, op
    
    ' delete the first pen.
    
    DeleteObject p

    ' Create the next pen for the bottom and right edges.

    If Reversed Then
        p = CreatePen(6, 1, GetSysColor(COLOR_BTNHIGHLIGHT))
    Else
        p = CreatePen(6, 1, GetSysColor(COLOR_BTNSHADOW))
    End If
    
    ' select the new pen.
    
    SelectObject hDC, p
        
    MoveToEx hDC, rcBounds.Right, rcBounds.Top, lp
    LineTo hDC, rcBounds.Right, rcBounds.Bottom
    LineTo hDC, rcBounds.Left, rcBounds.Bottom
    
    ' select the old pen.
    
    SelectObject hDC, op

    ' delete the second pen.

    DeleteObject p

End Sub

Public Function DrawCaptionSeparator(ByVal hDC As Long, rcBounds As RECT, m_Item As MenuItem)
    Dim b() As Byte, fState As Long, oldAlign As Long

    Dim x As Single, nx As Long
    
    If m_Item.Submenu.Count > 0 Then
        Draw3dSq hDC, rcBounds, True
    Else
        Draw3dSq hDC, rcBounds
    End If

    fState = DST_TEXT

    If (m_Item.Enabled = False) Then
        fState = fState Or DSS_DISABLED
    Else
        fState = fState Or DSS_NORMAL
    End If
        
    WideTOAscii m_Item.Caption, b
                    
    oldAlign = SetTextAlign(hDC, TA_CENTER)
    
    With rcBounds
                
        x = .Left + ((.Right - .Left) / 2)
        nx = Round(x, 0)
        
        DrawStateB hDC, 0, 0, b(0), Len(m_Item.Caption), nx, .Top, .Right - .Left, (.Bottom - .Top), fState
        
    End With
    
    SetTextAlign hDC, oldAlign
    
End Function

Public Function MeasureItem(ByVal hwnd As Long, ByVal lParam As Long) As Boolean
    On Error Resume Next
    
    Dim hDC As Long, v_hWnd As Long
    Dim m_Measure As MEASUREITEMSTRUCT
    Dim m_Item As MenuItem, v_Ln As Long
    Dim pSize As Size, tStr As String
    Dim hOldFont As Long, hGuiFont As Long
    
     ' Copy the data into something we can read.
    
    CopyMemory m_Measure, ByVal lParam, Len(m_Measure)
            
    ' Find the menu item in question (See FindMenuItem function)
    
    Set m_Item = FindMenuItem(m_Measure.ItemID)
            
    v_hWnd = hwnd
    
    If m_Item Is Nothing Then Exit Function
    
    If (m_Item.MenuId = m_Measure.ItemID) Then
        
        If GetMDIFrame(hwnd) Then
            hDC = GetDC(g_MDIWindows(0).hwnd)
        Else
            hDC = GetDC(hwnd)
        End If
        
        hGuiFont = GetItemFont(m_Item, hDC)
        hOldFont = SelectObject(hDC, hGuiFont)
        
        tStr = GetPlainCaption(m_Item.Caption)
        
        GetTextExtentPoint32 hDC, tStr, Len(tStr), pSize
        
        ReleaseDC v_hWnd, hDC
        
        If (m_Item.CaptionSeparator) Then
            m_Measure.itemHeight = pSize.cy
        Else
            m_Measure.itemHeight = MENU_HEIGHT
        End If
    
        If TypeOf m_Item.Parent Is Submenu Then
            m_Measure.itemWidth = ICONIC_SQ + pSize.cx
        Else
            m_Measure.itemWidth = pSize.cx
        End If
        
        If m_Item.Default = True Then
            m_Measure.itemWidth = m_Measure.itemWidth + (pSize.cx / 3)
        
        End If
        
        If Not m_Item.CaptionSeparator Then
            tStr = m_Item.Accelerator.AccelWord
            If tStr <> "" Then
                GetTextExtentPoint32 hDC, String(16, " ") + tStr, Len(tStr) + 16, pSize
                m_Measure.itemWidth = m_Measure.itemWidth + pSize.cx + (ICONIC_SQ + 10)
            End If
        
        End If
        ' Copy the new data back into the memory location so that the other calling
        ' procedures can see it.
            
        CopyMemory ByVal lParam, m_Measure, Len(m_Measure)
        MeasureItem = True
        SelectObject hDC, hOldFont
    End If
        
End Function

' IFont interface: {BEF6E003-A874-101A-8BBA-00AA00300CAB}

Public Function GetIFont(lfFont As LOGFONT) As StdFont

    Dim riid As GUID
    Dim vData As FONTDESC
    
    Dim vFont As StdFont
    Dim vName As String
    Dim hDC As Long
    
    With riid
    
        .Data1 = &HBEF6E003
        .Data2 = &HA874
        .Data3 = &H101A
        
        .Data4(0) = &H8B
        .Data4(1) = &HBA
        .Data4(2) = &H0
        .Data4(3) = &HAA
        .Data4(4) = &H0
        .Data4(5) = &H30
        .Data4(6) = &HC
        .Data4(7) = &HAB
        
    End With
    
        With vData
        
            .cbSize = Len(vData)
            
            hDC = GetDC(0&)
            .cySize = -(MulDiv(lfFont.lfHeight, 72, GetDeviceCaps(hDC, LOGPIXELSY)))
            ReleaseDC 0&, hDC
            
            .sWeight = lfFont.lfWeight
            
            .fStrikethrough = CLng(lfFont.lfStrikeOut)
            .fUnderline = CLng(lfFont.lfUnderline)
            .fItalic = CLng(lfFont.lfItalic)
            
            vName = BytesToString(lfFont.lfFaceName)
            .szName = StrPtr(vName)
            .sCharset = lfFont.lfCharSet
                    
        End With
        
        OleCreateFontIndirect vData, riid, vFont
    
        ' Sometimes things work this way!
        vFont.Size = vData.cySize
    
    Set GetIFont = vFont
    
End Function

Public Function PtrToString(ByVal lParam As Long) As String

    Dim i As Long, b As Byte
    
    i = lParam
    
    CopyMemory b, ByVal i, 1
    
    Do While b <> 0
    
        PtrToString = PtrToString + Chr(b)
            
        i = i + 1
        CopyMemory b, ByVal i, 1
        
    Loop

End Function

Public Function BytesToString(bArray() As Byte) As String
    Dim i As Long
    
    On Error Resume Next
    
    i = -1
    i = LBound(bArray)
    If i < 0 Then Exit Function
    
    For i = LBound(bArray) To UBound(bArray)
    
        If bArray(i) = 0 Then Exit For
        BytesToString = BytesToString + Chr(bArray(i))
    
    Next i
    
End Function

Public Function StringToBytes(sData As String, bArray() As Byte, Optional ByVal Fixed As Boolean = True)
    
    Dim i As Long
    
    If Not Fixed Then
        ReDim bArray(1 To (Len(sData) + 1))
    Else
        
        If (((UBound(bArray) - LBound(bArray)) + 1) < Len(sData)) Then Exit Function
        
    End If

    For i = 1 To Len(sData)
    
        bArray(i) = Asc(Mid(sData, i, 1))
    
    Next i
    
    bArray(i) = 0

End Function

Public Sub GetLogFont(IFont As StdFont, lfData As LOGFONT, hDC As Long)

    With lfData
        .lfCharSet = IFont.Charset
        .lfItalic = IFont.Italic
        .lfWeight = IFont.Weight
        
        If IFont.Bold Then
            .lfWeight = 700
        End If
        
        .lfUnderline = IFont.Underline
        StringToBytes IFont.Name, .lfFaceName
        .lfStrikeOut = IFont.Strikethrough
        .lfHeight = -(MulDiv(IFont.Size, GetDeviceCaps(hDC, LOGPIXELSY), 72))
            
    End With

End Sub

Public Function GetItemFont(varItem As MenuItem, hDC As Long) As Long
    On Error Resume Next
    
    Dim v_IFont As StdFont, lfData As LOGFONT, fontObj As Long
    
    If (varItem.Font Is Nothing) And (varItem.Parent.Font Is Nothing) Then
        Set v_IFont = g_SysFont
    ElseIf (varItem.Font Is Nothing) Then
        Set v_IFont = varItem.Parent.Font
    Else
        Set v_IFont = varItem.Font
    End If
    
    If varItem.Default = True Then
        v_IFont.Font.Bold = True
    End If
    
    fontObj = GetCurrentObject(hDC, OBJ_FONT)
    GetObject fontObj, 1, lfData
    
    GetLogFont v_IFont, lfData, hDC
    
    GetItemFont = CreateFontIndirect(lfData)
    
End Function

Public Sub PaintColorSelect(ByVal hDC As Long, rcBounds As RECT, ByVal ColorRef As Long)

    Dim lb_New As LOGBRUSH
    Dim hb_Paint As Long, hb_Old As Long
        
    '' In order to use FillRect with a bitplane color (not a system color)
    '' we need to create a brush with that color, and then fill using the brush.
    ''
    lb_New.lbColor = ColorRef
    
    hb_Paint = CreateBrushIndirect(lb_New)
    
    FillRect hDC, rcBounds, hb_Paint
    
    DeleteObject hb_Paint
    
End Sub


' function to draw the text with accelerator modifications

Public Function PaintAccelText(ByVal hDC As Long, rcBounds As RECT, v_Item As MenuItem, af_Info As LOGFONT, ByVal itemState As Long, Optional ByVal Underscore As Boolean = True)
    Dim pSize As Size, pSizeBefore As Size, pSizeAfter As Size
    Dim aKey As String, aPos As Long, aByte() As Byte
    Dim aStr As String, capStrBefore As String, CapStrAfter As String, scStr As String
    Dim a_Font As Long, v_Font As Long
                        
    ' Draw the item in enabled/disabled state.
                
    If InStr(1, v_Item.Caption, Chr(9)) <> 0 Then
        scStr = Mid(v_Item.Caption, InStr(1, v_Item.Caption, Chr(9)) + 1)
        aStr = GetPlainCaption(Mid(v_Item.Caption, 1, InStr(1, v_Item.Caption, Chr(9)) - 1))
        
    Else
        scStr = v_Item.Accelerator.AccelWord
        
        aStr = GetPlainCaption(v_Item.Caption)
    
    End If
    
    aPos = GetAccelPos(v_Item.Caption)
    
    If aPos <> 0 Then
        aKey = Mid(aStr, aPos, 1)
    
        If aPos > 1 Then
            capStrBefore = Mid(aStr, 1, aPos - 1)
        End If
        
        If aPos < Len(aStr) Then
            CapStrAfter = Mid(aStr, aPos + 1)
        End If
    Else
        capStrBefore = aStr
    End If
    
    If capStrBefore <> "" Then
        GetTextExtentPoint32 hDC, capStrBefore, Len(capStrBefore), pSizeBefore
        
        If itemState And ODS_DISABLED Then
            WideTOAscii capStrBefore, aByte
            DrawStateB hDC, 0, 0, aByte(0), Len(capStrBefore), rcBounds.Left, rcBounds.Top + 2, 0, 0, DSS_DISABLED + DST_TEXT
        Else
            TextOut hDC, rcBounds.Left, rcBounds.Top + 2, capStrBefore, Len(capStrBefore)
        End If
    End If
    
    If aPos <> 0 Then
        
        If (Underscore = True) Then
            If ((itemState And ODS_SELECTED) And (Not v_Item.HotTracked = True)) Or ((itemState And ODS_SELECTED) Xor ODS_SELECTED) Then
                af_Info.lfUnderline = 1
                a_Font = CreateFontIndirect(af_Info)
                v_Font = SelectObject(hDC, a_Font)
            End If
        End If
                        
        GetTextExtentPoint32 hDC, aKey, 1, pSize
        
        If itemState And ODS_DISABLED Then
            WideTOAscii aKey, aByte
            DrawStateB hDC, 0, 0, aByte(0), 1, rcBounds.Left + pSizeBefore.cx, rcBounds.Top + 2, 0, 0, DSS_DISABLED + DST_TEXT
        Else
            TextOut hDC, rcBounds.Left + pSizeBefore.cx, rcBounds.Top + 2, aKey, 1
        End If
        
        If a_Font <> 0 Then
            SelectObject hDC, v_Font
            DeleteObject a_Font
        End If
        
    End If
        
    If CapStrAfter <> "" Then
        GetTextExtentPoint32 hDC, CapStrAfter, Len(CapStrAfter), pSizeAfter
        
        If itemState And ODS_DISABLED Then
            WideTOAscii CapStrAfter, aByte
            DrawStateB hDC, 0, 0, aByte(0), Len(CapStrAfter), rcBounds.Left + pSizeBefore.cx + pSize.cx, rcBounds.Top + 2, 0, 0, DSS_DISABLED + DST_TEXT
        Else
            TextOut hDC, rcBounds.Left + pSizeBefore.cx + pSize.cx, rcBounds.Top + 2, CapStrAfter, Len(CapStrAfter)
        End If
    End If
    
    If scStr <> "" Then
        SetTextAlign hDC, TA_RIGHT
        
        If itemState And ODS_DISABLED Then
            WideTOAscii scStr, aByte
            DrawStateB hDC, 0, 0, aByte(0), Len(scStr), rcBounds.Right, rcBounds.Top + 2, 0, 0, DSS_DISABLED + DST_TEXT
        Else
            TextOut hDC, rcBounds.Right - (ICONIC_SQ + 10), rcBounds.Top + 2, scStr, Len(scStr)
        End If
        
        SetTextAlign hDC, TA_LEFT
    End If
        
    
End Function

Public Function DrawItem(lParam As Variant, Optional ByVal d_HotLight As Boolean = False) As Boolean
    On Error Resume Next
    
    Dim pRect As RECT
    Dim hDC As Long, hcdc As Long, v_DrawState As Long, lf_Data As LOGFONT
    Dim oldObj As Long, oldBgMode As Long, oldBgColor As Long, oldFgColor As Long
    Dim m_Draw As DRAWITEMSTRUCT, m_Item As MenuItem, v_Font As Long, v_OldFont As Long
    Dim m_SmIcon As Long, m_Err As Long, dBrush As LOGBRUSH, newBrush As Long
    
    If VarType(lParam) = vbLong Then
        hcdc = lParam
        CopyMemory m_Draw, ByVal hcdc, Len(m_Draw)
        Set m_Item = FindMenuItem(m_Draw.ItemID)
        If Not m_Item Is Nothing Then
            m_Item.SetLastDraw m_Draw
            m_Item.SetDCOrg m_Draw.hDC
        End If
    Else
        If TypeOf lParam Is MenuItem Then
            Set m_Item = lParam
            m_Item.GetLastDraw m_Draw
        End If
    End If
            
    ' Make sure we're working with an item on a menu that we handle...
    
    If m_Item Is Nothing Then Exit Function
    If m_Item.MenuId <> m_Draw.ItemID Then Exit Function
    If m_Item.Parent Is Nothing Then Exit Function
                                          
    '' Drawing canvas is a device context.
        
    hDC = m_Draw.hDC
    
    '' Get the appropriate font from the item data provided
    
    v_Font = GetItemFont(m_Item, hDC)
    v_OldFont = SelectObject(hDC, v_Font)
        
    '' Set the last known rectangle location of the menu
    '' item.
        
        ' If this is a popup menu (not a menu bar itme, then we have many
        ' more options available to us.
        
    If (TypeOf m_Item.Parent Is Submenu) And (Not m_Item.CaptionSeparator) Then
                                    
        pRect.Top = m_Draw.rcItem.Top + 1
        pRect.Left = m_Draw.rcItem.Left
        pRect.Bottom = m_Draw.rcItem.Bottom - 1
        pRect.Right = m_Draw.rcItem.Left + ICONIC_SQ + 2
        
        
        If (m_Item.Checked) Then
                
            ' Check mark supercedes a bitmap.
                        
                            
            ' Hot-tracked item checkeds are lightened.
                            
            If (m_Draw.itemState And ODS_DISABLED) Or (m_Item.HotTracked = True) Then
                FillRect hDC, pRect, COLOR_MENU + 1
            Else
            ' Do this  la Visual Basic IDE... darken the rectangle on select.
                                
                If (m_Draw.itemState And ODS_SELECTED) Then
                    FillRect hDC, pRect, COLOR_MENU + 1
                Else
                    FillRect hDC, pRect, COLOR_BTNHIGHLIGHT + 1
                End If
                
            End If
        
            ' Adjust the rectangular area by 1x1 pixel, since flood fill
            ' neglects the last row and column.
                            
            pRect.Bottom = m_Draw.rcItem.Bottom - 2
            pRect.Right = m_Draw.rcItem.Left + ICONIC_SQ + 1
                        
                            
            ' Draw the check mark with the appropriate status according to the state.
                            
            ' Disabled.
            If (m_Draw.itemState And ODS_DISABLED) Then
                                
                DrawCheck hDC, pRect, DRWCHK_DISABLED, m_Item.RadioGroup
                                
                ' Selected, Hot Tracked.
            ElseIf ((m_Draw.itemState And ODS_SELECTED) And (m_Item.HotTracked = True)) Then
                DrawCheck hDC, pRect, DRWCHK_SELECTED, m_Item.RadioGroup
                            
                ' Not Hottracked.
            Else
                DrawCheck hDC, pRect, DRWCHK_NORMAL, m_Item.RadioGroup
            End If
                            
            Draw3dSq hDC, pRect, True
                     
            ''' End If CheckMark
                    
            ' If it's a radio group, doesn't need to be checked to get a box.
                
        ElseIf (m_Item.RadioGroup = True) Then
                    
                      ' Set the picture area rectangle.
                    
            pRect.Top = m_Draw.rcItem.Top + 1
            pRect.Left = m_Draw.rcItem.Left
            pRect.Bottom = m_Draw.rcItem.Bottom - 1
            pRect.Right = m_Draw.rcItem.Left + ICONIC_SQ + 2
                            
            If (m_Draw.itemState And ODS_DISABLED) Or (m_Item.HotTracked = True) Then
                FillRect hDC, pRect, COLOR_MENU + 1
            Else
                If (m_Draw.itemState And ODS_SELECTED) Then
                    FillRect hDC, pRect, COLOR_MENU + 1
                Else
                    FillRect hDC, pRect, COLOR_BTNHIGHLIGHT + 1
                End If
            End If
                        
            pRect.Bottom = m_Draw.rcItem.Bottom - 2
            pRect.Right = m_Draw.rcItem.Left + ICONIC_SQ + 1
                    
            pRect.Bottom = m_Draw.rcItem.Bottom - 2
            pRect.Right = m_Draw.rcItem.Left + ICONIC_SQ + 1
                    
            Draw3dSq hDC, pRect, True
            
            ''' End If RadioGroup
                                                
            ' Not a radio group, not a check mark, does it have a picture?
                
        ElseIf (Not m_Item.Picture Is Nothing) Then
                                                     
            FillRect hDC, pRect, COLOR_MENU + 1
                        
            pRect.Bottom = m_Draw.rcItem.Bottom - 2
            pRect.Right = m_Draw.rcItem.Left + ICONIC_SQ + 1
                        
            ' Select enabled or disabled state.
                        
            If (m_Draw.itemState And ODS_DISABLED) Then
                v_DrawState = DSS_DISABLED
            Else
                v_DrawState = DSS_NORMAL
            End If
                    
            
            Select Case m_Item.Picture.Type
            
                Case 1
                    ' Picture is a Bitmap, tell the DrawState function to draw a bitmap.
                                
                    If v_DrawState And DSS_DISABLED Then
                        m_SmIcon = GetMonoPicture(m_Item.Picture.Handle, IMAGE_BITMAP)
                    Else
                        If (m_Item.MonoDefault) And ((m_Draw.itemState And ODS_SELECTED) = 0) Then
                                        
                            m_SmIcon = GetMonoPicture(m_Item.Picture.Handle, IMAGE_BITMAP)
                            v_DrawState = v_DrawState + DSS_MONO
                                    
                        Else
                                        
                            m_SmIcon = GetSmallPicture(m_Item.Picture.Handle, IMAGE_BITMAP)
                                
                        End If
                    End If
                    
                    v_DrawState = v_DrawState + DST_BITMAP
                            
                Case 3
                            
                    ' An icon, need to call GetSmallIcon to ensure that it's 16x16
                                
                    If v_DrawState And DSS_DISABLED Then
                        m_SmIcon = GetMonoPicture(m_Item.Picture.Handle, IMAGE_ICON)
                    Else
                        If (m_Item.MonoDefault) And ((m_Draw.itemState And ODS_SELECTED) = 0) Then
                                        
                            m_SmIcon = GetMonoPicture(m_Item.Picture.Handle, IMAGE_ICON)
                            v_DrawState = v_DrawState + DSS_MONO
                        Else
                                        
                            m_SmIcon = GetSmallPicture(m_Item.Picture.Handle, IMAGE_ICON)
                                
                        End If
                    End If
                    
                    v_DrawState = v_DrawState + DST_ICON
                        
            End Select
                        
            ' Draw the icon.
                        
            If m_SmIcon <> 0 Then
                m_Err = DrawState(hDC, newBrush, 0, m_SmIcon, 0, CInt(pRect.Left + 1), CInt(pRect.Top), 16, 16, v_DrawState)
            End If
                        
            ' Destroy any icon/bitmap we created.
                        
            If (m_SmIcon <> 0) And (m_SmIcon <> m_Item.Picture.Handle) Then
                If m_Item.Picture.Type = 3 Then
                    ' Type is icon, destroy as icon
                    
                    DestroyIcon m_SmIcon
                ElseIf m_Item.Picture.Type = 1 Then
                    ' Type is bitmap, destroy as object
                                
                    DeleteObject m_SmIcon
                End If
            End If
                        
            ' If the item is tracked, we put a box around the bitmap a la
            ' VB/O97
                        
            If (m_Draw.itemState And ODS_SELECTED) Then
                If (m_Draw.itemState And ODS_DISABLED) = False Then
                    Draw3dSq hDC, pRect
                End If
            End If
                      
                    
            ''' EndIf Picture
                      
        End If
                
    ''' EndIf Not CaptionSeparator
    
    End If
            
        
    '' That's just for the first 19x19 pixels!  whew!  now, to draw the text,
    '' and fill the rest of the rectangle.
        
    pRect.Left = m_Draw.rcItem.Left + ICONIC_SQ + 3
    pRect.Top = m_Draw.rcItem.Top
    pRect.Bottom = m_Draw.rcItem.Bottom
    pRect.Right = m_Draw.rcItem.Right
            
    If (m_Item.RadioGroup = False) And (m_Item.Picture Is Nothing) And (m_Item.Checked = False) Then
            
        ' If there is absolutely nothing to the left of the text, then the fill
        ' rectangle extends all the way.
            
        pRect.Left = m_Draw.rcItem.Left
    
    End If
        
        
        
    '' Selected items.
        
    If (m_Item.CaptionSeparator = False) And _
       ((m_Draw.itemState And ODS_SELECTED) Or _
        (m_Draw.itemState And ODS_FOCUS) Or _
        (m_Draw.itemState And ODS_HOTLIGHT)) Then
            
        If (m_Draw.itemState And ODS_SELECTED) And (Not (m_Draw.itemState And ODS_DISABLED)) And ( _
            Not m_Item.HotTracked) Then
            
            If (TypeOf m_Item.Parent Is EnhancedMenu) Or (m_Item.ClearHighlight = True) Then
                            
                pRect.Right = pRect.Right - 1
                pRect.Bottom = pRect.Bottom - 1
                            
                Draw3dSq hDC, pRect, True
                            
                pRect.Right = pRect.Right + 1
                pRect.Bottom = pRect.Bottom + 1
                
                oldBgColor = SetBkColor(hDC, GetSysColor(COLOR_MENU))
            
            Else
                                   
                If Not m_Item.ColorSelect Then
                            
                    FillRect hDC, pRect, COLOR_HIGHLIGHT + 1
                            
                    oldBgColor = SetBkColor(hDC, GetSysColor(COLOR_HIGHLIGHT))
                               
                Else
                            
                    PaintColorSelect hDC, pRect, m_Item.SelectColor
                            
                    oldBgColor = SetBkColor(hDC, m_Item.SelectColor)
                                                                                                        
                End If
            
            End If
                                        
        ElseIf (m_Draw.itemState And ODS_DISABLED) And _
                (m_Item.CaptionSeparator = False) Then
                    
                '' Disabled: we don't do a thing.
                
            If m_Draw.itemState And (ODS_SELECTED Or ODS_FOCUS) Then
                FillRect hDC, pRect, COLOR_BTNSHADOW + 1
                oldBgColor = SetBkColor(hDC, GetSysColor(COLOR_BTNSHADOW))
            Else
                FillRect hDC, pRect, COLOR_MENU + 1
                oldBgColor = SetBkColor(hDC, GetSysColor(COLOR_MENU))
            End If
                
        '' Hotlit menu item (menu bar items only)
        
        ElseIf (m_Draw.itemState And ODS_HOTLIGHT) Or (d_HotLight = True) Then
            
            If TypeOf m_Item.Parent Is EnhancedMenu Then
                pRect.Right = pRect.Right - 1
                pRect.Bottom = pRect.Bottom - 1
                    
                Draw3dSq hDC, pRect, False
                oldBgColor = SetBkColor(hDC, GetSysColor(COLOR_MENU))
                    
                pRect.Right = pRect.Right + 1
                pRect.Bottom = pRect.Bottom + 1
            End If
        
        End If
        
    Else
        
        '' Not selected.
            
        FillRect hDC, pRect, COLOR_MENU + 1
        oldBgColor = SetBkColor(hDC, GetSysColor(COLOR_MENU))
        
    End If
                
    '' Get the text rectangle, set the text attributes.
        
    pRect.Left = m_Draw.rcItem.Left + ICONIC_SQ + 3
        
    If m_Item.CaptionSeparator = True Then
        pRect.Left = m_Draw.rcItem.Left
                            
        If (m_Item.Submenu.Count = 0) Or ((m_Item.Submenu.Count > 0) And (m_Draw.itemState And ODS_SELECTED)) Then
            
            If (m_Item.HotTracked) Then
                
                oldFgColor = SetTextColor(hDC, m_Item.SelectColor)
                
            ElseIf m_Item.ColorSelect Then
                
                oldFgColor = SetTextColor(hDC, GetSysColor(COLOR_HIGHLIGHTTEXT))
                
                PaintColorSelect hDC, pRect, m_Item.SelectColor
                oldBgColor = SetBkColor(hDC, m_Item.SelectColor)
            
            ElseIf m_Item.Submenu.Count > 0 Then
                
                oldFgColor = SetTextColor(hDC, GetSysColor(COLOR_HIGHLIGHTTEXT))
                FillRect hDC, pRect, (COLOR_HIGHLIGHT + 1)
                oldBgColor = SetBkColor(hDC, GetSysColor(COLOR_HIGHLIGHT))
            
            End If
        
        Else
            
            oldFgColor = SetTextColor(hDC, GetSysColor(COLOR_MENUTEXT))
        
        End If
        
    ElseIf m_Draw.itemState And (ODS_GRAYED + ODS_DISABLED) Then
        
        '' Disabled or Grayed warrents grayed text.
        
        oldFgColor = SetTextColor(hDC, GetSysColor(COLOR_GRAYTEXT))
    
    ElseIf (m_Draw.itemState And ODS_SELECTED) Then
    
        If (m_Item.ClearHighlight = False) And (TypeOf m_Item.Parent Is Submenu) Then
            
            ' set the text highlight color.
            
            oldFgColor = SetTextColor(hDC, GetSysColor(COLOR_HIGHLIGHTTEXT))
        Else
            ' No highlight for Hotlight/Reverse Hotlight items.
            
            oldFgColor = SetTextColor(hDC, GetSysColor(COLOR_MENUTEXT))
        End If
        
    Else
        
        ' No highlight for unselected items.
        
        oldFgColor = SetTextColor(hDC, GetSysColor(COLOR_MENUTEXT))
    End If
                                    
    ' set the background mode to transparent for drawing text.
    
    oldBgMode = SetBkMode(hDC, TRANSPARENT)
            
    ' if the parent is popup menu, then the item is popup type,
    ' other wise it's a menu bar type.
                    
    ' if it's a Separator caption, the caption is drawn centered, so we use the whole recangle, anyway.
    
    If (TypeOf m_Item.Parent Is Submenu) And (m_Item.CaptionSeparator = False) Then
        m_Draw.rcItem.Left = m_Draw.rcItem.Left + ICONIC_SQ + 6
    ElseIf (m_Item.CaptionSeparator = False) Then
        m_Draw.rcItem.Left = m_Draw.rcItem.Left + 6
    Else
        m_Draw.rcItem.Left = m_Draw.rcItem.Left + 1
        m_Draw.rcItem.Right = m_Draw.rcItem.Right - 1
    End If
    
    ' If any special work needs to be done to the fontography of the
    ' selected item, then take care of that now.
        
    ' Get information about the current font handle selected
    ' for the device context of the menu frame.
        
    ' get the object LOGFONT structure from the font handle
        
    GetObject v_Font, Len(lf_Data), lf_Data
        
    ' change the weight to bold for default items.
        
    If (m_Item.CaptionSeparator = True) Or _
        (m_Item.Italic = True) Or _
        (m_Item.Default = True) Or _
        ((m_Item.HotTracked = True) And (m_Draw.itemState And ODS_SELECTED)) Then
        
        If m_Item.Default = True Then
            lf_Data.lfWeight = 700
        End If
        
        ' italicize the item if required.
        
        If m_Item.Italic = True Then
            lf_Data.lfItalic = 1
        End If
        
        ' HotTrack item if it's selected (underline, color).
        
        If ((m_Item.HotTracked = True) And (m_Draw.itemState And ODS_SELECTED)) Then
            lf_Data.lfUnderline = 1
            SetTextColor hDC, m_Item.SelectColor
        End If
        
        ' 600 = DEMi/SEMi Bold : For caption Separators.
        
        If m_Item.CaptionSeparator = True Then
            lf_Data.lfWeight = 600
            lf_Data.lfHeight = lf_Data.lfHeight * 0.9
            lf_Data.lfUnderline = 0
        End If
                    
        ' Create a new font handle from the modified LOGFONT
        ' and set it active on the drawing device.
        
        DeleteObject v_Font
        v_Font = CreateFontIndirect(lf_Data)
        SelectObject m_Draw.hDC, v_Font
        
    End If
    
    If m_Item.CaptionSeparator Then
                    
        m_Draw.rcItem.Bottom = m_Draw.rcItem.Bottom - 1
        DrawCaptionSeparator m_Draw.hDC, m_Draw.rcItem, m_Item
    
    Else
    
        PaintAccelText m_Draw.hDC, m_Draw.rcItem, m_Item, lf_Data, m_Draw.itemState, CBool((m_Draw.itemState And ODS_NOACCEL) Xor ODS_NOACCEL)
    
    End If
    
    ' If there was any special font modification made, there
    ' will be a value present, if not this value will be zero.
    
    SelectObject m_Draw.hDC, v_OldFont
    DeleteObject v_Font
    
    ' Set the device context back to the default state
    ' of background and foreground colors and painting modes.
    
    SetBkMode hDC, oldBgMode
    SetBkColor hDC, oldBgColor
    SetTextColor hDC, oldFgColor
    
    ' return success.
    
    DrawItem = True

End Function


Public Function MenuWndProc(ByVal hwnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
    On Error Resume Next
    
    Dim m_LastHit As MenuItem
    
    Dim v_ItemId As Long, m_Item As Object
    Dim v_LastId As Long
    
    Dim bar_Menu As EnhancedMenu
    Dim sub_Menu As Submenu
    Dim m_Menu As MenuItem
    
    Dim ht_Points As POINTAPI
    Dim hData As DRAWITEMSTRUCT
    
    Select Case uMsg
        
        Case WM_COMMAND
         
            ' Item was selected. ... Where?
            
            ' Find the menu passed by the calling procedure.
            ' To find the menu item id from the WM_COMMAND message,
            ' wParam And'd &Hffff  (low order word of wParam)
            
            Set m_Menu = FindMenuItem(wParam And &HFFFF)
            
            If Not m_Menu Is Nothing Then
            
                ' The superparent function finds the highest menu
                ' of the current instance.  Available only as a friend function
                ' This is a good idea, since the only menus associated with
                ' window handles are top level instances.
                
                m_Menu.MenuSelectEvent (wParam And &HFFFF)
                
                If TypeOf m_Menu.SuperParent Is EnhancedMenu Then
                
                    If m_Menu.SuperParent.Connected = True Then
                    
                        If Not (g_WindowList("H" + Hex(wParam)) Is Nothing) Then
                        
                            g_IsSetting = True
                            If g_WinVersion = Windows2000 Then
                                SendMessage m_Menu.SuperParent.Connection.hWndClient, WM_MDIACTIVATE, wParam, 0
                            Else
                                SendMessage wParam, WM_ACTIVATE, 1, 0
                            End If
                            g_IsSetting = False
                      
                        End If
                    
                    End If
                
                End If
            
            End If
        
        Case WM_MENUCHAR
        
            Set m_Item = g_MenuCol("H" + Hex(lParam))
            
            If TypeOf m_Item Is EnhancedMenu Then
            
                Set bar_Menu = m_Item
                v_ItemId = CmdBySelector(Chr(wParam And &HFF), m_Item)
                MenuWndProc = MNC_EXECUTE + v_ItemId
                Exit Function
                        
            ElseIf TypeOf m_Item Is Submenu Then
                
                Set sub_Menu = m_Item
                v_ItemId = CmdBySelector(Chr(wParam And &HFF), sub_Menu)
                MenuWndProc = MNC_EXECUTE + v_ItemId
                Exit Function
            
            End If
        
        Case WM_KEYDOWN
            
            If (CBool(GetKeyState(vbKeyControl) And &HFFFF0000)) Then
                g_KeyState = g_KeyState Or FCONTROL
            End If
            
            If (CBool(GetKeyState(vbKeyShift) And &HFFFF0000)) Then
                g_KeyState = g_KeyState Or FSHIFT
            End If
                    
        Case WM_KEYUP
        
            If g_KeyState Then
                Set m_Menu = KeyCodeToItem(hwnd, CInt(wParam), g_KeyState)
                    
                g_KeyState = 0
                If Not m_Menu Is Nothing Then
                    m_Menu.MenuSelectEvent m_Menu.MenuId
                    Exit Function
                End If
            End If
                     
        Case WM_DRAWITEM
            
            DrawItem lParam
            
            MenuWndProc = True
            
            ' Exit Function
            
        Case WM_MEASUREITEM
            
            MeasureItem hwnd, lParam
            
            MenuWndProc = True
            
            ' Exit Function
        
    End Select
    
    
    ' Window procedures enumerated by the class instances by a call to
    ' SetWindowHandle.
    
    v_ItemId = GetOldWndProc(hwnd)
    MenuWndProc = CallWindowProc(v_ItemId, hwnd, uMsg, wParam, lParam)
       
       
End Function

Public Sub RefreshWindowList(hwnd As Long)
    Dim v_Menu As Submenu, i As Long
        
    If (g_WindowListHandle <> 0) And (g_IsSetting = True) Then
    
        UpdateMDICaptions
        Set v_Menu = g_MenuCol("H" + Hex(g_WindowListHandle))
                
        If Not v_Menu Is Nothing Then
        
            v_Menu.WindowListClear
            
            i = -1
            i = LBound(g_MDIWindows)
            
            If (i >= 0) And (UBound(g_MDIWindows) >= 1) Then
            
                v_Menu.Add "-", , , , -1&
        
                For i = 1 To UBound(g_MDIWindows)
        
                    v_Menu.Add g_MDIWindows(i).Caption, , , , g_MDIWindows(i).hwnd
        
                Next i
        
            End If
        
        End If
        
    End If
    
    UpdateListChecks , hwnd
    
End Sub

Public Sub UpdateListChecks(Optional ByVal v_Activate As Long, Optional ByVal v_hWnd As Long)
    
    Dim v_Item As MenuItem, v_Menu As Submenu
        
    If (v_Activate = 0) And (v_hWnd <> 0) Then
        v_Activate = SendMessage(v_hWnd, WM_MDIGETACTIVE, 0, 0)
    End If
        
    If (g_WindowListHandle <> 0) Then
            
        Set v_Menu = g_MenuCol("H" + Hex(g_WindowListHandle))
            
        If Not v_Menu Is Nothing Then
                
            For Each v_Item In v_Menu.WindowListCol
            
                If v_Item.MenuId <> v_Activate Then
                    v_Item.Checked = False
                Else
                    v_Item.Checked = True
                End If
            
            Next v_Item
            
        End If
            
    End If
            
    Set v_Item = Nothing
    Set v_Menu = Nothing
            
End Sub

Public Function ClientWndProc(ByVal hwnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
    On Error Resume Next
    
    Dim v_Ptr As Long
    
    Dim v_Item As MenuItem
    Dim v_Draw As DRAWITEMSTRUCT
    
    Dim i As Long
    
    Select Case uMsg
            
        Case WM_MDISETMENU
            
            If Not g_IsSetting Then
                ClientWndProc = lParam
                Exit Function
            ElseIf g_WinVersion <> Windows2000 Then
                ClientWndProc = SetMenu(g_MDIWindows(0).hFrameWnd, wParam)
                DrawMenuBar g_MDIWindows(0).hFrameWnd
                Exit Function
            End If
                           
        Case WM_MDIACTIVATE
        
            v_Ptr = GetOldWndProc(hwnd)
            ClientWndProc = CallWindowProc(v_Ptr, hwnd, uMsg, wParam, lParam)
                                   
            If (g_WindowListHandle <> 0) And (g_IsSetting = True) Then
                g_IsSetting = True
                SendMessage hwnd, WM_MDIREFRESHMENU, 0, 0
                g_IsSetting = False
            End If
                                            
        Case WM_MDIREFRESHMENU
        
            RefreshWindowList hwnd
        
        Case WM_MDICREATE
                            
            v_Ptr = GetOldWndProc(hwnd)
            ClientWndProc = CallWindowProc(v_Ptr, hwnd, uMsg, wParam, lParam)
                                   
            AddMDIChild ClientWndProc, g_MDIWindows(0).hFrameWnd
                                                    
            If g_WindowListHandle <> 0 Then
                g_IsSetting = True
                SendMessage hwnd, WM_MDIREFRESHMENU, 0, 0
                g_IsSetting = False
            End If
            
            Exit Function
            
        Case WM_MDIDESTROY
        
            v_Ptr = GetOldWndProc(hwnd)
            ClientWndProc = CallWindowProc(v_Ptr, hwnd, uMsg, wParam, lParam)
                       
            RemoveMDIChild wParam
                        
            If g_WindowListHandle <> 0 Then
                g_IsSetting = True
                SendMessage hwnd, WM_MDIREFRESHMENU, 0, 0
                g_IsSetting = False
            End If
            
            Exit Function
            
    End Select
        
    v_Ptr = GetOldWndProc(hwnd)
    ClientWndProc = CallWindowProc(v_Ptr, hwnd, uMsg, wParam, lParam)

End Function

Public Function GetPlainCaption(ByVal vData As String) As String

    Dim i As Long, s As String, t As String
    
    For i = 1 To Len(vData)
        s = Mid(vData, i, 1)
        If s = "&" Then
            If Len(vData) > i Then
                If Mid(vData, i + 1, 1) = "&" Then
                    t = t + "&"
                End If
            End If
        Else
            t = t + s
        End If
    Next i
    
    GetPlainCaption = t
            
End Function

Public Function GetAccelPos(ByVal vData As String) As Long

    Dim i As Long, s As String, t As String
    
    For i = 1 To Len(vData)
        s = Mid(vData, i, 1)
        If s = "&" Then
            If Len(vData) > i Then
                If Mid(vData, i + 1, 1) <> "&" Then
                    GetAccelPos = Len(t) + 1
                End If
            End If
        Else
            t = t + s
        End If
    Next i
    
End Function


'' Copy one bitmap to another, changing the bitplanes from 24 to 1.

Public Function GetMonoPicture(ByVal v_hPicture As Long, ByVal v_Type As Integer) As Long
    Dim x As Long
    
    
    If v_Type = IMAGE_ICON Then
        
        x = CopyImage(v_hPicture, IMAGE_ICON, 16, 16, &H41)
        
    Else
    
        x = CopyImage(v_hPicture, IMAGE_BITMAP, 16, 16, &H41)
    
    End If
    
    GetMonoPicture = x
    
End Function

'' Function to get a 16x16 representation of the picture

Public Function GetSmallPicture(ByVal v_hPicture As Long, ByVal v_Type As Integer) As Long

    GetSmallPicture = CopyImage(v_hPicture, v_Type, 16, 16, &H44)
    
End Function


Public Function WideTOAscii(ByVal wString As String, aString() As Byte)

    Dim b() As Byte, i As Integer
    
    ReDim aString(Len(wString))
    
    For i = 1 To Len(wString)
    
        aString(i - 1) = Asc(Mid(wString, i, 1))

    Next i
    
    aString(i - 1) = 0
    
End Function



' Find a menu given an item ID.

' Because it is possible to have more than one menu on a window or as part of this thread,
' we will assign all menu identifiers incrementally from the first menu created
' on the first window destination to the last menu item in the last Submenu in the last
' window, using a global identifier that is incremented by every created instance of
' the EnhancedMenu object.

Public Function FindMenuItem(ByVal v_ItemId As Long) As MenuItem
        
    On Error Resume Next

    ' Calling late-bound objects in callback processes is not a good idea
    ' and is somewhat unsuccessful.  Call each according to its type.

    Dim v_Menu As Object
    Dim m_Menu As MenuItem
    Dim i As Long
    
    ' Iterate the collection, return on success.
    
    For i = 1 To g_MenuCol.Count
        
        Set v_Menu = Nothing
        Set v_Menu = g_MenuCol(i)
        
        Set m_Menu = SearchRecursive(v_Menu, v_ItemId)
        
        If Not m_Menu Is Nothing Then
            Set FindMenuItem = m_Menu
            Exit Function
        End If
    
    Next i
                
    Set FindMenuItem = Nothing
    
End Function

    ' The search function will read the whole data collection.
    ' If it is textual, then it will search every menu and every
    ' Items of every menu and so forth looking for a matching key.
    ' If it's an integer value, then we assume, that it refers directly to the
    ' MenuId assigned by the GetCtrlID function in EnhancedMenu
        
Public Function SearchRecursive(v_Menu As Object, ByVal varIndexKey As Variant) As MenuItem
    On Error Resume Next
    
    Dim i As Long
    
    Dim vntIndex As Long, vsKey As String
    Dim v_wlCol As Collection, v_wlMenu As Submenu
    
    Dim v_Item As MenuItem, v_SearchItem As MenuItem
    
    Set SearchRecursive = Nothing
    
    '' Make sure that we run only if passed a vaid object parameter!
    
    If ((Not TypeOf v_Menu Is EnhancedMenu) And (Not TypeOf v_Menu Is Submenu)) _
        Or (v_Menu Is Nothing) Then
           
            Exit Function
            
    ElseIf TypeOf v_Menu Is Submenu Then
        Set v_wlMenu = v_Menu
    
    End If
    
    ''' If the vartype is an integer we do not assume that it
    ''' is an Items identifier but a menu control identifier.
    
    ''' Note: Use Items() to get an Items by the collection index.
    
    If Not v_Menu(varIndexKey) Is Nothing Then
        Exit Function
    
    ElseIf v_Menu.Count = 0 Then
        Exit Function
    End If
    
    '' If the menu, itself, is the item in question, we return nothing
    '' since the search function returns only valid menu items (not valid menus).
        
    Select Case VarType(varIndexKey)
    
        Case vbLong
        
            vntIndex = varIndexKey
            If v_Menu.MenuId = varIndexKey Then Exit Function
    
        Case vbString
        
            vsKey = varIndexKey
            If v_Menu.Key = varIndexKey Then Exit Function
    
        Case Else
        
            Exit Function
            
    End Select
                    
        
    Select Case VarType(varIndexKey)
    
        Case vbLong
                
            For Each v_Item In v_Menu
    
                If v_Item.MenuId = vntIndex Then
                    Set SearchRecursive = v_Item
                    Exit Function
                End If
            
            Next v_Item
            
            If (TypeOf v_Menu Is Submenu) Then
                
                Set v_wlCol = v_wlMenu.WindowListCol
                
                If (v_wlMenu.WindowList = True) Then
                
                    For Each v_Item In v_wlCol
                        
                        If v_Item Is Nothing Then Exit For
                        
                        If v_Item.MenuId = vntIndex Then
                            Set SearchRecursive = v_wlCol(v_Item.Key)
                            Exit Function
                        End If
                        
                    Next v_Item
                End If
            End If
            
        Case vbString
                            
            For Each v_Item In v_Menu
                If v_Item.Key = vsKey Then
                
                    Set SearchRecursive = v_Item
                    Exit Function
            
                End If
            Next v_Item
            
            If (TypeOf v_Menu Is Submenu) Then
                
                Set v_wlCol = v_wlMenu.WindowListCol
                
                If (v_wlMenu.WindowList = True) Then
                
                    For Each v_Item In v_wlCol
                        
                        If v_Item Is Nothing Then Exit For
                        
                        If v_Item.Key = vsKey Then
                            Set SearchRecursive = v_wlCol(v_Item.Key)
                            Exit Function
                        End If
                        
                    Next v_Item
                End If
            End If
            
    End Select
                
    For Each v_Item In v_Menu
                
        If v_Item.Submenu.Count > 0 Then
            Set v_SearchItem = SearchRecursive(v_Item.Submenu, varIndexKey)
            
            If Not (v_SearchItem Is Nothing) Then
                Set SearchRecursive = v_SearchItem
                Exit Function
            End If
        
        End If
                        
    Next v_Item
                
    If (TypeOf v_Menu Is Submenu) Then
        Set v_wlCol = v_wlMenu.WindowListCol
        
        If v_wlMenu.WindowList = True Then
            For Each v_Item In v_wlCol
                        
                If v_Item Is Nothing Then Exit For
                        
                If v_Item.Submenu.Count > 0 Then
                    Set v_SearchItem = SearchRecursive(v_Item.Submenu, varIndexKey)
                End If
                
                If Not v_SearchItem Is Nothing Then
                    Set SearchRecursive = v_SearchItem
                    Exit Function
                End If
                        
            Next v_Item
        End If
    End If
                
End Function

Public Function StrTok(ByVal v_Str As String, ByVal vTok As String) As String
    Static vOldStr As String
    Dim i As Long, l As Long, c As String, d As String
    
        
    If Not v_Str = vbNullString Then
        vOldStr = v_Str
    End If
    
    If vOldStr = vbNullString Then Exit Function
    
    If Mid(vOldStr, 1, Len(vTok)) = vTok Then
        vOldStr = Mid(vOldStr, Len(vTok) + 1)
        StrTok = ""
        Exit Function
    End If
        
    For i = 1 To Len(vOldStr)
        c = Mid(vOldStr, i, Len(vTok))
        If c = vTok Then
            StrTok = d
            vOldStr = Mid(vOldStr, i + Len(vTok))
            Exit Function
        Else
            c = Mid(vOldStr, i, 1)
            d = d + c
        End If
    Next i
    
    StrTok = d
    vOldStr = vbNullString
    
End Function

Public Function GetOldWndProc(ByVal hwnd As Long) As Long
    On Error Resume Next
    Dim x  As Long
    
    x = -1
    x = UBound(g_Handles)
    
    If x = -1 Then Exit Function

    For x = LBound(g_Handles) To UBound(g_Handles)
        If g_Handles(x).hwnd = hwnd Then
            GetOldWndProc = g_Handles(x).Proc
            Exit Function
        End If
    Next x

End Function

Private Function MakePoints(ByVal lParam As Long) As POINTAPI

    Dim p As POINTAPI
    
    p.x = (lParam And &HFFFF&)
    p.y = lParam / &H10000
    
    MakePoints = p
    
End Function

